/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui.metaaudit;

import java.awt.*;
import java.util.*;
import java.io.*;
import javax.swing.table.*;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.file.FileNode;
import org.greenstone.gatherer.msm.Metadata;
import org.greenstone.gatherer.msm.MSMUtils;

/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.1 
 */
public class MetaAuditModel
    extends AbstractTableModel {

    private ArrayList elements;
    private MetadataTableCache data;

    public MetaAuditModel(FileNode records[]) {
	this.elements = new ArrayList(Gatherer.c_man.getCollection().msm.getElements());
	Collections.sort(this.elements, MSMUtils.METADATA_COMPARATOR);
	this.data = new MetadataTableCache(elements.size() + 1);
	// Work our way through the record array provided, adding files, skipping blanks and recursively searching directories.
	for(int i = 0; records != null && i < records.length; i++) {
	    if(records[i] != null) {
		addRecord(records[i]);
	    }
	}
    }
    public Class getColumnClass(int column) {
	return ArrayList.class;
    }
    public int getColumnCount() {
	return elements.size() + 1;
    }
    public String getColumnName(int column) {
	String result;
	if(column == 0) {
	    result = Dictionary.get("AuditTable.File");
	}
	else {
	    result = elements.get(column - 1).toString();
	}
	return result;
    }
    public ArrayList getColumnValues(int column) {
	ArrayList column_values = new ArrayList();
	if(0 <= column && column < getColumnCount()) {
	    for(int i = 0; i < data.size(); i++) {
		ArrayList values = (ArrayList) data.get(i, column);
		for(int j = 0; j < values.size(); j++) {
		    column_values.add(values.get(j).toString());
		}
	    }
	}
	return column_values;
    }
    public int getRowCount() {
	return data.size();
    }
    public Object getValueAt(int row, int column) {
	Object result = "";
	if(0 <= row && row < data.size() && 0 <= column && column < elements.size() + 1) {
	    result = data.get(row, column);
	}
	return result;
    }

    public void reset() {
    }

    private void addRecord(FileNode record) {
	if(record.isLeaf()) {
	    File file = record.getFile();
	    data.newRow();
	    data.put(0, file.getName());
	    ArrayList metadatum = Gatherer.c_man.getCollection().gdm.getMetadata(file);
	    for(int i = 0; i < metadatum.size(); i++) {
		Metadata metadata = (Metadata) metadatum.get(i);
		int index = elements.indexOf(metadata.getElement())+1;
		if (index!=0) {
		    // some elements are hidden and dont appear in teh table
		    data.put(index, metadata.getValueNode());
		}
	    }
	}
	else {
	    for(int i = 0; i < record.getChildCount(); i++) {
		addRecord((FileNode)record.getChildAt(i));
	    }
	}
    }

    private class MetadataTableCache {
	private ArrayList data;
	private ArrayList current_row[];
	private int columns;
	private RowComparator row_comparator = new RowComparator();
	public MetadataTableCache(int columns) {
	    this.columns = columns;
	    this.data = new ArrayList();
	}

	public ArrayList get(int row, int column) {
	    ArrayList array_list[] = (ArrayList[]) data.get(row);
	    return array_list[column];
	}

	public void newRow() {
	    if(current_row != null) {
		for(int i = 0; i < columns; i++) {
		    Collections.sort(current_row[i], row_comparator);
		}
	    }
	    current_row = new ArrayList[columns];
	    for(int i = 0; i < columns; i++) {
		current_row[i] = new ArrayList();
	    }
	    data.add(current_row);
	}

	public void put(int column, Object value) {
	    current_row[column].add(value);
	}

	public int size() {
	    return data.size();
	}
    }

    private class RowComparator
	implements Comparator {
	public int compare(Object o1, Object o2) {
	    return o1.toString().compareTo(o2.toString());
	}
	public boolean equals(Object obj) {
	    return toString().equals(obj.toString());
	}
    }
}
